class Human {
	constructor(game, type, weapon, x, y, r){
		this.game = game;
		
		// Rendering
		this.mainHand = Math.random() > 0.5 ? "left" : "right";
		this.map = game.atlas.players[type][weapon];
		this.texture = game.textures.players;
		
		this.map = this.map[this.mainHand] || this.map["left"];
		this.map = this.map[Math.floor(Math.random() * this.map.length)];
		
		this.width  = 6;
		this.height = 4;
		
		// Physics
		this.r = r || 0;
		this.x = x;
		this.y = y;
		this.vx = 0;
		this.vy = 0;
		this.vr = 0;
		
		// Gravity
		this.gravity = 1;
		
		if(this.gravity){
			this.ax = 9;
			this.ay = 9;
			this.vxDamp = 1.3;
			this.vyDamp = 1.3;
			this.vrDamp = 1.3;
		}
		
		else{
			this.ax = 0.75;
			this.ay = 0.75;
			this.vxDamp = 1.01;
			this.vyDamp = 1.01;
			this.vrDamp = 1.01;
		}
		
		// Human
		this.weaponCooldown = 0;
	}
	
	
	update(delta, input){
		var actionMoveLeft  = false, // input.keys[65]   || input.keys[37],
			actionMoveRight = false, // input.keys[68]   || input.keys[39],
			actionMoveUp    = false, // input.keys[87]   || input.keys[38],
			actionMoveDown  = false, // input.keys[83]   || input.keys[40],
			actionShoot     = false; // input.mouse.down || input.keys[88];
		
		// Sort some input stuff
		if(actionMoveLeft && actionMoveRight) actionMoveLeft = actionMoveRight = false;
		if(actionMoveUp   && actionMoveDown ) actionMoveUp   = actionMoveDown  = false;
		
		var didMove = false;
		
		if(this.gravity){
			var across = (actionMoveLeft || actionMoveRight) && (actionMoveUp || actionMoveDown),
				mult = across ? 1/1.414 : 1;
			
			if(actionMoveLeft ) this.vx -= delta * this.ax * mult, didMove = true;
			if(actionMoveRight) this.vx += delta * this.ax * mult, didMove = true;
			if(actionMoveUp   ) this.vy -= delta * this.ay * mult, didMove = true;
			if(actionMoveDown ) this.vy += delta * this.ay * mult, didMove = true;
			
		}
		
		// In space, arrow keys move the player
		else{
			if(actionMoveLeft ) this.vx -= delta * this.ax, didMove = true;
			if(actionMoveRight) this.vx += delta * this.ax, didMove = true;
			if(actionMoveUp   ) this.vy -= delta * this.ay, didMove = true;
			if(actionMoveDown ) this.vy += delta * this.ay, didMove = true;
		}
		
		// Physics
		this.x += this.vx;
		this.y += this.vy;
		this.r += this.vr;
		
		this.vx /= this.vxDamp;
		this.vy /= this.vyDamp;
		
		// Left mouse click shoots
		if(actionShoot && this.weaponCooldown === 0){
			// TODO: fire projectiles
		}
		
		// Footstep Particles
		if(didMove && this.game.canShowParticle(10)){
			var particle = new FootstepParticle(this.game, this.game.textures.world.tundra, this.x, this.y, this.r);
			this.game.scene.addParticle(particle);
		}
		
		this.weaponCooldown -= delta;
		if(this.weaponCooldown<0) this.weaponCooldown = 0;
	}
	
	
	draw(context, textures, atlas){
		var texture = this.texture,
			map = this.map;
		
		context.save();
		context.translate(this.x, this.y);
		context.rotate(this.r/180*Math.PI);
		
		context.drawImage(this.texture, map[0], map[1], map[2], map[3], -this.width/2, -this.height/2 - (map[3] - this.height) - 1, map[2], map[3]);

		context.restore();
	}
}
